@@ -12,9 +12,13 @@
 from __future__ import print_function
 from __future__ import division
 
+import re
+import numpy as np
+
 from past.utils import old_div
 from clasp import click
 import matplotlib.colors as mplc
+from matplotlib.patches import Wedge
 
 import clasp.click_ext as clk
 
@@ -36,9 +40,9 @@ main = plot # backwards compatibility
 coloropt = [
           click.option('-fcol', default=0.0, type=float,
                        help="colormap position for first color"),
-          click.option('-colors', default='med',
+          click.option('-colors', default='viridis',
                        callback=clk.color_inp,
-                       help="cmap name* or space seperated list of rgb tuples "
+                       help="cmap name or space seperated list of rgb tuples "
                        "0,0,0 120,120,120 etc. if fewer than # series picks ."
                        "from gradient includes ru_colors: 'blu', 'org', 'pur'"
                        ", 'red', 'blg', 'mag', 'brn', 'grn', 'yel', 'ggr', "
@@ -65,14 +69,16 @@ shared = coloropt + [
                        help="foreground color"),
           click.option('-bg', default="white",
                        help="background color"),
-          click.option('--legend/--no-legend', default=True,
+          click.option('--legend/--no-legend', default=False,
                        help="include legend in plot"),
+          click.option('-xrotate', default='auto', callback=clk.char0, type=click.Choice(['auto', 'horiz', 'vert', 'a', 'h', 'v'], case_sensitive=False),
+                       help="orientation of x-axis labels"),
           click.option('-comment', default='#',
                        help='regex "^[{}].*" indicating comment line '
                        'indicator'),
-          click.option('-labelweight', default='bold', type=click.Choice(['ultralight', 'light', 'normal', 'regular', 'book', 'medium', 'roman', 'semibold', 'demibold', 'demi', 'bold', 'heavy', 'extra bold', 'black']),
+          click.option('-labelweight', default='ultralight', type=click.Choice(['ultralight', 'light', 'normal', 'regular', 'book', 'medium', 'roman', 'semibold', 'demibold', 'demi', 'bold', 'heavy', 'extra bold', 'black']),
                        help="text weight for axes labels"),
-          click.option('-width', default=10.5, type=float,
+          click.option('-width', default=5.0, type=float,
                        help="image width"),
           click.option('-height', default=5.0, type=float,
                        help="image height"),
@@ -102,9 +108,9 @@ shared = coloropt + [
 
 # used by bar, scatter, histo
 sharedA = [
-           click.option('--xgrid/--no-xgrid', default=False,
+           click.option('--xgrid/--no-xgrid', default=True,
                         help="plot x grid lines"),
-           click.option('--ygrid/--no-ygrid', default=False,
+           click.option('--ygrid/--no-ygrid', default=True,
                         help="plot y grid lines"),
            click.option('-yticks', type=int,
                         help="number of y-ticks/gridlines"),
@@ -117,7 +123,7 @@ sharedA = [
                         "file name and index or --header option"),
            click.option('--rows/--no-rows', default=False,
                         help="get data rows instead of columns"),
-           click.option('-ecolors', default="1,1,1", callback=clk.color_inp,
+           click.option('-ecolors', default="viridis", callback=clk.color_inp,
                         help="marker edge colors"),
            click.option('-epositions', default=None,
                         callback=clk.split_float,
@@ -374,6 +380,30 @@ def histo(ctx, dataf, **kwargs):
     return 'histo', kwargs, ctx
 
 
+def wedge_mark(d, s):
+    t1 = d - s/2
+    t2 = d + s/2
+    wd = Wedge(0, 1, t1, t2).get_path()
+    return ruplot.WedgeMark(wd, np.array([t1, d, t2]))
+
+
+def parse_mark(ctx, param, s):
+    if s in [None, 'None', 'none']:
+        result = None
+    else:
+        result = []
+        for part in s.split():
+            try:
+                a = int(part)
+            except ValueError:
+                if re.match(r"w\.\d+\.\d+", part):
+                    a = wedge_mark(*[int(i) for i in part.split(".")[1:]])
+                else:
+                    a = part
+            result.append(a)
+    return result
+
+
 @plot.command('scatter')
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-autox', default=None,
@@ -423,11 +453,11 @@ def histo(ctx, dataf, **kwargs):
               help="label y-axis by percentile with N bins")
 @click.option('--reverse/--no-reverse', default=False,
               help="reverse order of y data (sometimes useful with -autox)")
-@click.option('-ms', default="0", callback=clk.split_float,
+@click.option('-ms', default="2", callback=clk.split_float,
               help="marker size if fewer than # series uses last")
 @click.option('-mew', default="0", callback=clk.split_float,
               help="marker edge width")
-@click.option('-mrk', default='o', callback=clk.split_str,
+@click.option('-mrk', default='o', callback=parse_mark,
               help="marker style if fewer than # series loops")
 @click.option('-criteria',
               help="plot markers based on criteria (either x=VAL or y=VAL)"
@@ -485,7 +515,9 @@ def scatter(ctx, dataf, **kwargs):
                 ax2 = ruplot.tick_from_arg(ax2, xs, [ys[i] for i in kwargs['y2']], a4, kwargs)
                 fig.sca(ax2)
                 kwargs['axes'] = axes
-            ax = ruplot.tick_from_arg(ax, xs, ys + areas, a4, kwargs)
+                ax = ruplot.tick_from_arg(ax, xs, [ys[i] for i in range(len(ys)) if i not in kwargs['y2']] + areas, a4, kwargs)
+            else:
+                ax = ruplot.tick_from_arg(ax, xs, ys + areas, a4, kwargs)
             a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
             a5['positions'] = kwargs['epositions']
@@ -542,6 +574,18 @@ def scatter(ctx, dataf, **kwargs):
               help="linewidth for median line")
 @click.option('-series', default=1,
               help="number of series to plot data in")
+@click.option('-whis', default='1.5', callback=clk.split_float,
+              help="adapted from matplotlib documentation: As a single number,"
+                   "determines the reach of the whiskers to the beyond the "
+                   "first and third quartiles. In other words, where IQR is "
+                   "the interquartile range (Q3-Q1), the upper whisker will "
+                   "extend to last datum less than Q3 + whis*IQR). "
+                   "Similarly, the lower whisker will extend to the first "
+                   "datum greater than Q1 - whis*IQR. Beyond the whiskers, "
+                   "data are considered outliers and are plotted as individual "
+                   "points. As two numbers, set this to an ascending sequence "
+                   "of percentile (e.g., [5, 95]) to set the whiskers at "
+                   "specific percentiles of the data. ignored if --no-fliers")
 @click.option('--clbg/--no-clbg', default=True,
               help="median line uses -bg")
 @click.option('-fillalpha', default=1.0,
@@ -566,13 +610,16 @@ def scatter(ctx, dataf, **kwargs):
               help="plot mean line or median line")
 @click.option('--notch/--no-notch', default=False,
               help="plot notch for 95\% confidence interval")
+@click.option('--fliers/--no-fliers', default=True,
+              help="plot notch for 95\% confidence interval")
 @click.option('--inline/--no-inline', default=False,
               help="keep boxes inline")
 @click.option('--header/--no-header', default=False,
               help="indicates that data has a header row to get "
               "series labels (overridden by labels)")
 @clk.shared_decs(shared)
-def box(dataf, **kwargs):
+@click.pass_context
+def box(ctx, dataf, **kwargs):
     """
     create boxplot from data files.
     """
@@ -596,6 +643,11 @@ def box(dataf, **kwargs):
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
             a6 = mgr.kwarg_match(ruplot.plot_box, kwargs)
             a6.pop('labels', None)
+            a6['xlabels'] = xlabels
+            if len(a6['whis']) == 1:
+                a6['whis'] = a6['whis'][0]
+            else:
+                a6['whis'] = (a6['whis'][0], a6['whis'][-1])
             ax, handles = ruplot.plot_box(ax, ys, labels, cmap,
                                             axext['ydata'], **a6)
             if kwargs['outf']:
@@ -608,7 +660,7 @@ def box(dataf, **kwargs):
             raise
         except Exception as ex:
             clk.print_except(ex, kwargs['debug'])
-    return 'box', kwargs
+    return 'box', kwargs, ctx
 
 
 @plot.command('violin')
@@ -641,6 +693,8 @@ def box(dataf, **kwargs):
               help="scale factor to apply to kernel bandwidth selector (1 uses scotts rule)")
 @click.option('-fillalpha', default=.5,
               help="alpha for fill color (matches line color)")
+@click.option('-weightlimit', default=0.0,
+              help="exclude values below weight limit from plot (useful to remove artificial whiskers)")
 @click.option('--xheader/--no-xheader', default=False,
               help="indicates that data has a header column to get x-axis "
               "labels (overridden by xlabels)")
@@ -661,13 +715,16 @@ def box(dataf, **kwargs):
               help="plot mean dot")
 @click.option('--median/--no-median', default=True,
               help="plot median line")
+@click.option('--fliers/--no-fliers', default=False,
+              help="plot outliers as fliers")
 @click.option('--inline/--no-inline', default=False,
               help="keep violins inline")
 @click.option('--header/--no-header', default=False,
               help="indicates that data has a header row to get "
               "series labels (overridden by labels)")
 @clk.shared_decs(shared)
-def violin(dataf, **kwargs):
+@click.pass_context
+def violin(ctx, dataf, **kwargs):
     """
     create violinplot from data files.
     """
@@ -712,7 +769,7 @@ def violin(dataf, **kwargs):
             raise
         except Exception as ex:
             clk.print_except(ex, kwargs['debug'])
-    return 'violin', kwargs
+    return 'violin', kwargs, ctx
 
 
 @plot.command('previewpal')
@@ -824,9 +881,12 @@ def colors(**kwargs):
             if kwargs['mpal'] is not None:
                 cmap = ruplot.get_colors(kwargs['mpal'])
                 click.echo("\ncolor palette: {} in {} steps:".format(kwargs['mpal'], kwargs['mpaldiv']))
+                mpc = []
                 for i in range(kwargs['mpaldiv']):
                     c = cmap.to_rgba(i/(kwargs['mpaldiv'] - 1.0))
-                    click.echo("{:03d},{:03d},{:03d}".format(*[int(j*255) for j in c[0:3]]))
+                    mpc.append("({},{},{})".format(*[int(j*255) for j in c[0:3]]))
+                print(f"[{','.join(mpc)}]")
+                print(" ".join(mpc).replace("(", "").replace(")", ""))
                 if kwargs['radf']:
                     click.echo("\n ungammad 0-1:")
                     for i in range(kwargs['mpaldiv']):
