@@ -30,7 +30,6 @@ from matplotlib.transforms import Bbox
 import clasp.script_tools as mgr
 from hdrstats import hdrstats as hs
 
-
 daycount = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]
 
 
@@ -69,7 +68,7 @@ def tick_from_arg(ax, xs, ys, a4, kwargs):
         a4['tcol'] = kwargs['fg']
     except Exception:
         pass
-    for i in ['polar', 'stacked', 'polarautp']:
+    for i in ['polar', 'stacked', 'polarauto']:
         try:
             kwargs[i] = kwargs[i]
         except Exception:
@@ -127,19 +126,29 @@ def get_labels(dataf, labs, a1, ycnt, xheader=False, xlabels=None,
     xlabs = get_user_labels(xlabs, xlabels)
     labs = get_user_labels(labs, labels)
     if rows:
-        if drange is not None:
-            la = [labs[i] for i in drange]
-        else:
-            la = labs[:ycnt]
+        # if drange is not None:
+        #     try:
+        #         la = [labs[i] for i in drange]
+        #     except IndexError:
+        #         la = []
+        # else:
+        la = labs[:ycnt]
         labs = []
         for i in y_vals:
             try:
                 labs.append(xlabs[i[1]])
             except TypeError:
-                labs.append(xlabs[i])
+                try:
+                    labs.append(xlabs[i])
+                except IndexError:
+                    pass
+                
         xlabs = la
-    elif drange is not None:
-        xlabs = [xlabs[i] for i in drange]
+    # elif drange is not None:
+    #     try:
+    #         xlabs = [xlabs[i] for i in drange]
+    #     except IndexError:
+    #         xlabs = []
     return labs, xlabs
 
 def ax_limits(x):
@@ -228,12 +237,25 @@ def plot_setup(fg='black', bg='white', polar=False, areaonly=False, params={}, *
     return ax, fig
 
 
+def series_ticks(ax, locs, xlabels, xrotate='a'):
+    ax.set_xticks(locs)
+    rots = dict(a='auto', h='horizontal', v='vertical')
+    ro = rots[xrotate]
+    if ro == 'auto':
+        if max([len(i) for i in xlabels]) > 10:
+            ro = 'vertical'
+        else:
+            ro = 'horizontal'
+    ax.set_xticklabels(xlabels, rotation=ro)
+    for t in ax.xaxis.get_ticklines():
+        t.set_visible(False)
+
 def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
           xgrid=True, ygrid=True, xscale='linear', yscale='linear',
           annualx=False, dayy=False, pery=False, ticklines=False, pph=1,
           bottom=None, bg='white', xlabels=None, dpy=365, hpd=24, sh=0,
           polar=False, xticks=None, yticks=None, labelweight='ultralight',
-          matchxy=False, **kwargs):
+          matchxy=False, xrotate='a', **kwargs):
     """
     setup ticks/axes for plot
 
@@ -265,8 +287,8 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
     ticklines: bool
         if not grid, show ticklines
     """
-    xmin = xdata[0]
-    xmax = xdata[-1]
+    xmin = min(xdata)
+    xmax = max(xdata)
     ymin = min(ydata)
     ymax = max(ydata)
     if matchxy:
@@ -295,18 +317,20 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
     elif pery:
         ax = ticks_y_per(ax, ydata, pery, tcol)
     for tick in ax.yaxis.get_major_ticks():
-        tick.label.set_fontsize(7)
-        tick.label.set_color(tcol)
+        tick.label1.set_fontsize(7)
+        tick.label1.set_color(tcol)
     for t in ax.yaxis.get_ticklines() + ax.yaxis.get_ticklines(minor=True):
         t.set_visible(ticklines and not ygrid)
         t.set_color(tcol)
     for tick in ax.xaxis.get_major_ticks():
-        tick.label.set_fontsize(7)
-        tick.label.set_color(tcol)
+        tick.label1.set_fontsize(7)
+        tick.label1.set_color(tcol)
     for t in ax.xaxis.get_ticklines() + ax.xaxis.get_ticklines(minor=True):
         t.set_visible(ticklines and not xgrid)
         t.set_color(tcol)
-    if bottom is not None and bottom > 0:
+    if bottom is None:
+        bottom = 0
+    if bottom > 0:
         ylim = [ymin, ymax]
         ylimb = [i+bottom for i in ylim]
         ax.set_ylim(bottom=ylim[0], top=ylimb[1])
@@ -318,16 +342,10 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
             ax.set_xticklabels(xlabels)
             ax.set_xlim(left=xmin, right=xmax)
         else:
-            ax.set_xticks(np.arange(xmin+old_div(inc,2), xmax+old_div(inc,2), inc))
-            if max([len(i) for i in xlabels]) > 10:
-                ro = 'vertical'
-            else:
-                ro = 'horizontal'
-            ax.set_xticklabels(xlabels, rotation=ro)
-            for t in ax.xaxis.get_ticklines():
-                t.set_visible(False)
+            locs = np.arange(xmin+old_div(inc,2), xmax+old_div(inc,2), inc)
+            series_ticks(ax, locs, xlabels, xrotate=xrotate)
     if polar:
-        if bottom is not None and bottom > 0:
+        if bottom > 0:
             ci = Circle((0, 0), zorder=2, radius=bottom, fill=True,
                         linewidth=0.3, edgecolor=tcol, facecolor=bg,
                         transform=ax.transData._b)
@@ -336,10 +354,9 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
                     fill=False, linewidth=0, edgecolor='blue',
                     facecolor='blue', transform=ax.transData._b)
         ax.add_artist(ci)
-        ax.yaxis.set_ticklabels([])
+        if not ygrid:
+            ax.yaxis.set_ticklabels([])
         ax.axes.set_ylabel("")
-    if bottom is None:
-        bottom = 0
     if yticks is not None:
         if yscale != 'log':
             ax.set_yticks(np.append(np.arange(bottom+ymin, ymax+bottom,
@@ -354,7 +371,10 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
             ax.yaxis.grid(ygrid, which='minor')
     if xticks is not None:
         if xscale != 'log':
-            ax.set_xticks(np.append(np.arange(xmin, xmax, (xmax-xmin)/xticks),xmax))
+            if xticks == 0:
+                ax.set_xticks([])
+            else:
+                ax.set_xticks(np.append(np.arange(xmin, xmax, (xmax-xmin)/xticks),xmax))
         else:
             decs = np.ceil(np.log10(xmax) - np.log10(xmin))
             maj = np.power(10, np.log10(xmin) + np.arange(decs+1))
@@ -417,7 +437,7 @@ def plot_graph(fig, saveimage, width=5, height=5, bg='white', fg='black',
         ax.axes.set_ylabel("")
         ax.xaxis.set_ticklabels([])
         ax2 = fig.add_axes([0, 0, 1, 1])
-        plargs = dict(bbox_inches=None, aspect='auto', pad_inches=0)
+        plargs = dict(bbox_inches=0, aspect='auto', pad_inches=0)
     else:
         ax2 = fig.add_subplot(1, 1, 1, label='background')
         plargs = dict(bbox_inches='tight')
@@ -459,6 +479,13 @@ def get_colors(cmap, step=None, positions=None, funcs=[], **kwargs):
     """
     efuncs = funcs + [cmap_from_mpl, cmap_from_clipt, cmap_from_list]
     e = []
+    try:
+        alpha = int(cmap.rsplit('_', 1)[1])
+    except (IndexError, ValueError, AttributeError):
+        alpha = None
+    else:
+        cmap0 = cmap
+        cmap = cmap.rsplit('_', 1)[0]
     for func in efuncs:
         try:
             colormap = func(cmap, step, positions)
@@ -468,6 +495,11 @@ def get_colors(cmap, step=None, positions=None, funcs=[], **kwargs):
             continue
     else:
         raise ValueError("\n\n".join(e))
+    if alpha is not None:
+        clist = colormap.cmap(np.arange(colormap.cmap.N))
+        clist[:,-1] = alpha/100
+        cmap = colors.LinearSegmentedColormap.from_list(cmap0, clist)
+        colormap.cmap = cmap
     return colormap
 
 
@@ -585,6 +617,13 @@ def color_inc_i(fcol, i, n, step):
     return cinc
 
 
+class WedgeMark(object):
+
+    def __init__(self, wedge, d):
+        self.mrk = wedge
+        self.scale = np.max(np.abs(np.concatenate((np.sin(d*np.pi/180), np.cos(d*np.pi/180)))))
+
+
 def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
                  mrk='o', step=None, fcol=0.0, mew=0.0, emap=None, estep=None,
                  flipxy=False, cs=None, cmin=None, cmax=None, y2=None,
@@ -595,7 +634,6 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
     for i in range(len(ys)):
         if i >= nlab:
             labels.append("series{:02d}".format(i))
-    handles = []
     if emap is None:
         emap = colormap
     if y2 is None:
@@ -612,10 +650,9 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
         j = 0
         cvals = []
         for i in range(len(ys)):
-            if get_nth(ms, i) > 0:
-                cvals.append(get_nth(cs, j))
-                j += 1
-            else:
+            try:
+                cvals.append(cs[i])
+            except IndexError:
                 cvals.append(None)
     for i, (x, y, l, cs) in enumerate(zip(xs, ys, labels, cvals)):
         if i in y2:
@@ -642,6 +679,11 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
         c = colormap.to_rgba(cinc)
         ecinc = color_inc_i(fcol, i, len(ys), estep)
         mec = emap.to_rgba(ecinc)
+        if hasattr(mka, "scale"):
+            msa = msa * mka.scale
+            mkt = mka.mrk
+        else:
+            mkt = mka
         if i < len(areas):
             if len(areas[i]) < 2:
                 ya2 = axT.axes.get_ylim()[0]
@@ -650,27 +692,23 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
             axT.fill_between(x, areas[i][0], ya2, alpha=falpha, color=c, zorder=-1,
                             linestyle='--', linewidth=lwa/2)
         if cs is not None:
+            colormap.norm.vmax = cmax
+            colormap.norm.vmin = cmin
             plotargs = {'linewidth': lwa, 's': msa**2, 'label': l,
-                        'marker': mka, 'cmap': colormap.cmap, 'linewidth': mewa,
-                        'vmin': cmin, 'vmax': cmax, 'c': cs, 'edgecolors': mec,
+                        'marker': mkt, 'cmap': colormap.cmap, 'linewidth': mewa,
+                        'c': cs, 'edgecolors': mec,
                         'norm': colormap.norm}
             plotargs.update(kwargs)
             axT.scatter(x, y, **plotargs)
-            if legend:
-                pc = cmx.ScalarMappable(norm=colormap.norm, cmap=colormap.cmap)
-                if polar:
-                    add_colorbar(fig, colormap, axes = [1, 0.11, .02, 0.77], orientation='vertical')
-                else:
-                    add_colorbar(fig, colormap, axes = [.92, 0.11, .02, 0.77], orientation='vertical')
         elif msd is not None:
             plotargs = {'linewidth': lwa, 's': msa**2, 'label': l,
-                        'marker': mka, 'linewidth': mewa, 'c': [c],
+                        'marker': mkt, 'linewidth': mewa, 'c': [c],
                         'edgecolors': mec}
             plotargs.update(kwargs)
             axT.scatter(x, y, **plotargs)
         else:
             plotargs = {'linewidth': lwa, 'markersize': msa, 'label': l,
-                        'marker': mka, 'color': c, 'mfc': c, 'mec': mec,
+                        'marker': mkt, 'color': c, 'mfc': c, 'mec': mec,
                         'mew': mewa}
             plotargs.update(kwargs)
             if flipxy:
@@ -681,6 +719,13 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
             copts = {'linewidth': 0, 'mfc': c, 'mew': 0,
                      'markersize': lwa*2.5, 'marker': 'o'}
             plot_criteria(ax, x, y, criteria, flipxy, copts)
+    if legend and cs is not None:
+        if polar:
+            add_colorbar(fig, colormap, axes=[1, 0.11, .02, 0.77],
+                         orientation='vertical')
+        else:
+            add_colorbar(fig, colormap, axes=[.92, 0.11, .02, 0.77],
+                         orientation='vertical')
     handles, _ = ax.get_legend_handles_labels()
     try:
         handles2, _ = ax2.get_legend_handles_labels()
@@ -766,9 +811,10 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
     return ax, handles
 
 
-def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
+def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x', whis=1.5,
              mew=0.5, ms=3.0, lw=1.0, fcol=0.0, clw=1.0, clbg=True, fillalpha=1.0, notch=False,
-             series=1, bg='white', inline=False, mean=False, **kwargs):
+             series=1, sgap=0.5, bg='white', inline=False, mean=False,
+             fliers=True, xlabels=None, **kwargs):
     """adds box plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(series):
@@ -777,6 +823,8 @@ def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
     nlab = len(labels)
     chunksize = int(len(data)/series)
     handles = []
+    if series == 1:
+        sgap = 0
     for i in range(series):
         cinc = color_inc_i(fcol, i, nlab, step)
         c = colormap.to_rgba(cinc)
@@ -800,21 +848,33 @@ def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
             x = np.arange(2, len(data), series)
             sw = series
         else:
-            x = np.arange(i, len(data), series)
+            x = np.arange(i, len(data), series) + np.linspace(0, sgap*(chunksize - 1), chunksize)
             sw = 1
+        if fliers:
+            whis = whis
+        else:
+            whis = (0, 100)
         boxplot = ax.boxplot(data[i*chunksize:i*chunksize+chunksize], notch=notch,
-                             patch_artist=True, widths=rwidth*sw, bootstrap=1000,
+                             patch_artist=True, widths=rwidth*sw, bootstrap=1000, whis=whis,
                              positions=x , manage_ticks=False, showmeans=mean, meanline=not notch,
                              **plotargs)
         handles.append(Patch(color=c, label=labels[i]))
-    lims = ax.dataLim
-    ax.set_xlim(left=lims.x0-rwidth, right=lims.x1+rwidth)
+    if inline:
+        ax.set_xlim(left=-.5, right=len(data)-.5)
+        tickx = np.arange(chunksize) - .5
+    else:
+        ax.set_xlim(left=-.5 - sgap/2, right=len(data) + (chunksize - .5) * sgap - .5)
+        tickx = np.linspace(series/2, len(data) + (chunksize - 1) * sgap - series/2, chunksize) - .5
+    if xlabels is None or len(xlabels) == 0:
+        xlabels = np.arange(chunksize).astype(str)
+    series_ticks(ax, tickx, xlabels, xrotate='a')
     return ax, handles
 
 
 def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0, kernelwidth=.5,
                 clw=1.0, clbg=True, fcol=0.0, fillalpha=1.0, median=True, conf=None, confm=None,
-                series=1, bg='white', inline=False, mean=False, weights=None, **kwargs):
+                series=1, bg='white', inline=False, mean=False, weights=None, weightlimit=0.0,
+                fliers=False, **kwargs):
     """adds violin plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(series):
@@ -831,10 +891,11 @@ def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0,
         else:
             medianc = c
         if inline:
-            x = np.arange(2, len(data), series)
+            x = np.arange(1, len(data), series)
             sw = series
         else:
-            x = np.arange(i, len(data), series)
+            bwidth = 1 - (1-rwidth)/2
+            x = np.arange(i*bwidth + 1 - bwidth, len(data), series)
             sw = 1
         ds = np.array(data[i*chunksize:i*chunksize+chunksize])
         if weights is not None:
@@ -842,26 +903,45 @@ def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0,
         else:
             ws = (None,) * len(ds)
         vstats = []
+        flies = []
         for d, w in zip(ds, ws):
-            vstats.append(hs.kernel(d, w=w, n=1000, bws=kernelwidth))
+            vs = hs.kernel(d, w=w, n=1000, bws=kernelwidth, t=weightlimit)
+            if fliers:
+                qr = np.quantile(d, (.25, .75))
+                iqr = (qr[1] - qr[0]) * 1.5
+                filt = np.logical_and(d >= qr[0] - iqr, d <= qr[1] + iqr)
+                # df = np.maximum(np.minimum(d, qr[1] + iqr), qr[0] - iqr)
+                df = d[filt]
+                vs['min'] = np.min(df)
+                vs['max'] = np.max(df)
+                flies.append(d[np.logical_not(filt)])
+                # print(np.stack((np.arange(len(d))[np.logical_not(filt)], flies[-1])).T)
+            vstats.append(vs)
         vplot = ax.violin(vstats, showmeans=mean, showmedians=median,
                           widths=rwidth*sw, positions=x)
         if confm is not None:
             bstats = hs.conf_box(ds, ws, confm)
             plotargs = {
                 'boxprops' : {'linewidth':clw, 'color':c, 'linestyle':'--', 'dash_joinstyle':'miter'},
-                'medianprops' : {'linewidth':0,},
-                'whiskerprops': {'linewidth':0,}
+                'medianprops' : {'linewidth': 0},
+                'whiskerprops': {'linewidth': 0},
             }
-            blot = ax.bxp(bstats, widths=rwidth*sw/2, positions=x, showfliers=False, showcaps=False, **plotargs)
+            ax.bxp(bstats, widths=rwidth*sw/4, positions=x, showfliers=False, showcaps=False, **plotargs)
         if conf is not None:
             bstats = hs.quant_box(ds, ws, conf)
+            if fliers:
+                for j in range(len(bstats)):
+                    bstats[j]['fliers'] = flies[j]
             plotargs = {
                 'boxprops' : {'linewidth':lw, 'color':c},
                 'medianprops' : {'linewidth':0,},
-                'whiskerprops': {'linewidth':0,}
+                'whiskerprops': {'linewidth':0,},
+                'flierprops': {'marker': 'x', 'markeredgecolor': c,
+                               'markeredgewidth': lw*0.5,
+                               'markersize': lw*4
+                               }
             }
-            blot = ax.bxp(bstats, widths=rwidth*sw/4, positions=x, showfliers=False, showcaps=False, **plotargs)
+            ax.bxp(bstats, widths=rwidth*sw/2, positions=x, showfliers=fliers, showcaps=False, **plotargs)
         for vp in vplot['bodies']:
             vp.set_facecolor(c)
             vp.set_alpha(fillalpha)
@@ -1076,7 +1156,105 @@ class CliptColors(object):
             'yel': [(188, 184, 0), (216, 220, 40),
                     (255, 255, 0), (255, 255, 153)],
             'ggr': [(79, 98, 40), (117, 146, 60),
-                    (194, 214, 154), (215, 228, 188)]
+                    (194, 214, 154), (215, 228, 188)],
+            'pm3d': [(0,0,0), (0,0,0), (0,0,0), (0,0,0),
+                     (66.0867532,7.96888909,104.060351),
+                     (92.488467,14.9637761,187.732777),
+                     (109.047472,14.9637761,233.803835),
+                     (121.939217,14.9637761,252.950118),
+                     (132.358234,14.9637761,253.40607),
+                     (140.057572,14.9637761,241.683544),
+                     (147.257293,14.9637761,221.341144),
+                     (153.248603,14.9637761,196.634698),
+                     (158.356722,10.920175,168.020405),
+                     (163.076322,17.99353,138.40435),
+                     (167.272654,17.99353,108.725664),
+                     (170.97227,22.695682,78.6556833),
+                     (174.59649,26.4459575,50.4598788),
+                     (177.75777,29.6460755,22.695682),
+                     (180.870487,32.4744741,10.920175),
+                     (183.593905,35.045542,10.920175),
+                     (186.421679,40.6276297,14.9637761),
+                     (188.384223,40.6276297,14.9637761),
+                     (190.963397,44.5014275,14.9637761),
+                     (192.870679,48.0160978,14.9637761),
+                     (195.379053,51.2462187,14.9637761),
+                     (197.251041,54.2413161,14.9637761),
+                     (199.086349,57.056982,14.9637761),
+                     (200.901576,59.7150308,14.9637761),
+                     (202.697332,64.6381163,14.9637761),
+                     (203.883973,66.9356291,14.9637761),
+                     (205.648552,71.2780323,14.9637761),
+                     (207.409999,73.3293562,14.9637761),
+                     (208.564557,77.2475565,14.9637761),
+                     (209.711497,79.1152419,14.9637761),
+                     (211.417928,82.7045137,14.9637761),
+                     (212.546426,86.1202011,14.9637761),
+                     (213.66778,89.3764982,14.9637761),
+                     (214.782116,90.9464589,14.9637761),
+                     (216.45484,94.0002246,14.9637761),
+                     (217.552076,98.3695502,14.9637761),
+                     (218.642712,101.192683,14.9637761),
+                     (219.726858,103.890284,14.9637761),
+                     (220.266531,106.506341,14.9637761),
+                     (221.341144,109.047472,14.9637761),
+                     (222.409533,112.731129,14.9637761),
+                     (223.471799,115.108729,14.9637761),
+                     (224.541542,118.568474,14.9637761),
+                     (225.067395,120.837634,14.9637761),
+                     (226.114702,124.107292,14.9637761),
+                     (227.15622,127.276711,14.9637761),
+                     (227.674836,130.354137,14.9637761),
+                     (228.707839,132.358234,14.9637761),
+                     (229.735272,135.297865,14.9637761),
+                     (230.246927,138.186961,14.9637761),
+                     (231.266164,140.981729,14.9637761),
+                     (231.773767,143.711545,14.9637761),
+                     (232.293004,146.380511,14.9637761),
+                     (233.301529,148.992321,14.9637761),
+                     (233.803835,152.413044,14.9637761),
+                     (234.804582,154.903525,14.9637761),
+                     (235.303041,157.34686,14.9637761),
+                     (235.800236,159.745492,14.9637761),
+                     (236.790871,162.877981,14.9637761),
+                     (237.284328,165.200236,14.9637761),
+                     (237.776557,167.464767,14.9637761),
+                     (238.267566,170.428165,14.9637761),
+                     (239.24596,172.610768,14.9637761),
+                     (239.73336,175.48854,14.9637761),
+                     (240.232026,178.292961,14.9637761),
+                     (240.717033,180.362028,14.9637761),
+                     (241.200869,183.077261,14.9637761),
+                     (241.683544,185.74501,14.9637761),
+                     (242.645439,188.384223,14.9637761),
+                     (243.124675,190.963397,14.9637761),
+                     (243.60278,193.501432,14.9637761),
+                     (244.079761,196.000135,14.9637761),
+                     (244.555627,198.476842,14.9637761),
+                     (245.030384,200.901576,14.9637761),
+                     (245.50404,203.291691,14.9637761),
+                     (245.976602,205.648552,14.9637761),
+                     (246.448077,207.988239,14.9637761),
+                     (246.918472,210.282154,14.9637761),
+                     (247.387794,213.107988,14.9637761),
+                     (247.868042,215.350888,14.9637761),
+                     (248.335212,217.552076,14.9637761),
+                     (248.80133,220.266531,14.9637761),
+                     (249.266402,222.409533,14.9637761),
+                     (249.730435,224.541542,14.9637761),
+                     (250.193436,227.15622,14.9637761),
+                     (250.655411,229.222246,14.9637761),
+                     (251.116366,231.773767,14.9637761),
+                     (251.576308,234.30485,14.9637761),
+                     (251.576308,236.296176,14.9637761),
+                     (252.035243,238.757364,14.9637761),
+                     (252.493178,241.200869,14.9637761),
+                     (252.950118,243.124675,14.9637761),
+                     (253.40607,245.50404,14.9637761),
+                     (253.86104,247.868042,14.9637761),
+                     (254.315033,250.193436,14.9637761),
+                     (254.315033,252.035243,14.9637761),
+                     (254.768055,254.315033,14.9637761)]
         }
         self.cdict = {}
         for k, v in self.cdict255.items():
@@ -1182,3 +1360,28 @@ def get_nth_loop(li, i):
         return li[i % len(li)]
     except Exception:
         return li
+
+
+def quick_scatter(xs, ys, ax=None, fig=None, outf=None, colors='viridis', tkwargs=None, ckwargs=None,
+                  polar=False, labels=None, legend=False, width=5, height=5, areaonly=False, **kwargs):
+    if tkwargs is None:
+        tkwargs = {}
+    if ckwargs is None:
+        ckwargs = {}
+    if labels is None:
+        labels = []
+    if ax is None or fig is None:
+        ax, fig = plot_setup(polar=polar, areaonly=areaonly)
+    cmap = get_colors(colors, **ckwargs)
+    ax, handles, handles2 = plot_scatter(fig, ax, xs, ys, labels, cmap, polar=polar, legend=legend, **kwargs)
+    if 'xdata' not in tkwargs:
+        tkwargs['xdata'] = flat(xs)
+    if 'ydata' not in tkwargs:
+        tkwargs['ydata'] = flat(ys)
+    tkwargs['polar'] = polar
+    ticks(ax, **tkwargs)
+    if outf is None:
+        return ax, fig
+    else:
+        plot_graph(fig, outf, width=width, height=height, legend=legend, handles=handles, handles2=handles2, polar=polar, areaonly=areaonly)
+
